﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Forms;
using foxit;
using foxit.pdf;
using foxit.addon.xfa;
using foxit.common.fxcrt;
using PDFReader_Demo_CSharp.Common;
using PDFReader_Demo_CSharp.Interface.Windows;

namespace PDFReader_Demo_CSharp.UI
{
    public class XFADocProvider : foxit.addon.xfa.DocProviderCallback
    {
        private WeakReference reader_doc_weakref_ = null;
        private const UInt32 default_hightlight_color_ = 0x50FF0000;


        public XFADocProvider(CPDFReaderDoc pReaderDoc)
        {
            reader_doc_weakref_ = new WeakReference(pReaderDoc);
        }

        CPDFReaderDoc GetDocument()
        {
            return reader_doc_weakref_.IsAlive ? reader_doc_weakref_.Target as CPDFReaderDoc : null;
        }
        

        public override void DisplayCaret(int page_index, bool is_visible, RectF rect)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return;

            main_form.DisplayCaret(page_index, is_visible, ref rect);
        }

        public override void Dispose()
        {
            base.Dispose();
        }

        public override void ExportData(XFADoc doc, string file_path)
        {

        }

        public override int GetCurrentPage(XFADoc doc)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return -1;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return -1;

            return main_form.GetCurPageIndex();
        }

        public override uint GetHighlightColor(XFADoc doc)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return 0;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return 0;

            return default_hightlight_color_;
        }

        public override bool GetPopupPos(int page_index, float min_popup, float max_popup, RectF rect_widget, RectF inout_rect_popup)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return false;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null || main_form.Viewer == null)
                return false;

            Control view_control = main_form.Viewer as Control;
            if (view_control == null)
                return false;

            Rectangle client_rect = view_control.ClientRectangle;

            foxit.common.fxcrt.RectF new_rect = new RectF();
            main_form.Viewer.WindowToDoc(page_index, client_rect, ref new_rect);

            inout_rect_popup.left = 0;
            if (rect_widget.right > new_rect.right)
            {
                inout_rect_popup.left -= rect_widget.right - new_rect.right;
                inout_rect_popup.right -= rect_widget.right - new_rect.right;
            }

            if (Math.Abs(new_rect.bottom - rect_widget.bottom) >= max_popup)
            {
                inout_rect_popup.top = Math.Abs(rect_widget.bottom - rect_widget.top);
                inout_rect_popup.bottom = inout_rect_popup.top - max_popup;
                return true;
            }

            if (rect_widget.top - new_rect.top >= max_popup)
            {
                inout_rect_popup.top = -max_popup;
                inout_rect_popup.bottom = inout_rect_popup.top - max_popup;
                return true;
            }

            if (new_rect.bottom - rect_widget.bottom >= min_popup)
            {
                inout_rect_popup.top = Math.Abs(rect_widget.top - rect_widget.bottom);
                inout_rect_popup.bottom = inout_rect_popup.top - (new_rect.bottom - rect_widget.bottom);
                return true;
            }

            if (rect_widget.top - new_rect.top >= min_popup)
            {
                inout_rect_popup.top = -(inout_rect_popup.top - new_rect.top);
                inout_rect_popup.bottom = inout_rect_popup.top - (rect_widget.top - new_rect.top);
                return true;
            }

            return false;
        }

        public override string GetTitle(XFADoc doc)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return "";
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return "";

            return "XFA Title";
        }

        public override void GotoURL(XFADoc doc, string url)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return;

            WindowsAPI.ShellExecute(main_form.Handle, "Open", url, null, null, WindowsAPI.ShowCommands.SW_SHOW);
        }

        public override void ImportData(XFADoc doc, string file_path)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return;

        }

        public override void InvalidateRect(int page_index, RectF rect, DocProviderCallback.InvalidateFlag flag)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return;

            main_form.ReDraw();
        }

        public override void PageViewEvent(int page_index, DocProviderCallback.PageViewEventType page_view_event_type)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return;

            Graphics graphics = Graphics.FromHwnd(IntPtr.Zero);
            reader_doc.UpdatePDFPages((int)graphics.DpiX, (int)graphics.DpiY, main_form.GetRotate(), true);
            main_form.UpdatePageSize();
        }

        public override bool PopupMenu(int page_index, foxit.common.fxcrt.PointF rect_popup)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return false;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null || main_form.Viewer == null)
                return false;

            System.Drawing.Point point = new System.Drawing.Point();
            main_form.Viewer.DocToWindow(page_index, rect_popup, ref point);

            main_form.ShowXfaPopup(point);

            return false;
        }

        public override void Print(XFADoc doc, int start_page_index, int end_page_index, int options)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return;

            main_form.OnPrint(start_page_index, end_page_index, options);
        }

        public override void Release()
        {

        }

        public override void SetCurrentPage(XFADoc doc, int current_page_index)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return;
            MainForm main_form = reader_doc.GetView();
            main_form.GoToPage(current_page_index);
            main_form.UpdateCurPage(current_page_index);
        }

        public override void SetFocus(XFAWidget xfa_widget)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return;

            main_form.SetFocusXFAWidget(xfa_widget);
        }

        public override bool SubmitData(XFADoc doc, string target, DocProviderCallback.SubmitFormat format, DocProviderCallback.TextEncoding text_encoding, string content)
        {
            CPDFReaderDoc reader_doc = GetDocument();
            if (reader_doc == null)
                return false;
            MainForm main_form = reader_doc.GetView();
            if (main_form == null)
                return false;

            MessageBox.Show("The target is : \n" + target);

            return true;
        }
    }
}